/* ======================================================================
**  TOptions.cpp - Исходный файл модуля формы параметров библиотеки.
**  Является частью программы Клён-библиотека, распространяемой под
**  Стандартной общественной лицензией, версии 2 (GNU GPL v2).
** ======================================================================
**  Copyleft 2009 - 2013 by Lab Free Technologies <admin@LabFreeTech.org>
** ==============================================================================
**  Copyleft 2009 - 2013 Лаборатория свободных технологий <admin@LabFreeTech.org>
** ==============================================================================
*/

#include "TFormOptions.h"


TFormOptions::TFormOptions(QDialog *pwgt /* = 0*/): QDialog(pwgt),
    ui(new Ui::TFormOptions)
{
    ui->setupUi(this);
}

TFormOptions::~TFormOptions()
{
    delete ui;
}

void TFormOptions::initVariables()
{
    if (ui->cbLanguage->currentIndex() == 0)
        FileOptions.setLanguageProgram(TFileOptions::LANGUAGE_RUSSIAN);
    else
        FileOptions.setLanguageProgram(TFileOptions::LANGUAGE_ENGLISH);

    FileOptions.setStateStandardToolbar(ui->cbStandardToolbar->isChecked());
    FileOptions.setStateEditToolbar(ui->cbEditToolbar->isChecked());
    FileOptions.setStateFindToolbar(ui->cbFindToolbar->isChecked());
    FileOptions.setStateFormatToolbar(ui->cbFormatToolbar->isChecked());
    FileOptions.setSavePasswordBook(ui->cbSavePasswordBook->isChecked());
    FileOptions.setPathOptionsLocally(ui->cbPathOptionsLocally->isChecked());
    FileOptions.setPathOptions(ui->lePathOptions->text());
    FileOptions.setPathStatistics(ui->lePathStatistics->text());
    FileOptions.setRememberLastOpened(ui->cbLastOpened->isChecked());
    FileOptions.setPathLastLibrary(ui->lePathLibrary->text());
    FileOptions.setPathProgramShelkTest(ui->lePathShelkTest->text());
}

void TFormOptions::on_cbLastOpened_clicked()
{
    ui->lePathLibrary->setEnabled(ui->cbLastOpened->isChecked());
    ui->pbPathLibrary->setEnabled(ui->cbLastOpened->isChecked());
}

void TFormOptions::on_cbPathOptionsLocally_clicked()
{
    ui->lePathOptions->setEnabled(!ui->cbPathOptionsLocally->isChecked());
    ui->pbPathOptions->setEnabled(!ui->cbPathOptionsLocally->isChecked());
}

void TFormOptions::on_bbOptions_clicked(QAbstractButton *button)
{
    //Нажата кнопка "Восстановить умолчания"
    if (button == ui->bbOptions->button(QDialogButtonBox::RestoreDefaults))
    {
        ui->cbSavePasswordBook->setChecked(true);
        ui->cbLastOpened->setChecked(true);
        on_cbLastOpened_clicked();
        ui->lePathLibrary->clear();
        ui->cbPathOptionsLocally->setChecked(true);
        on_cbPathOptionsLocally_clicked();
        ui->lePathOptions->clear();
        ui->lePathStatistics->setText(QDir::homePath());
        ui->cbStandardToolbar->setChecked(true);
        ui->cbEditToolbar->setChecked(false);
        ui->cbFormatToolbar->setChecked(false);
        ui->cbFindToolbar->setChecked(false);
        ui->cbLanguage->setCurrentIndex(0);
        ui->lePathShelkTest->clear();
    }

    //Нажата кнопка "Отменить"
    if (button == ui->bbOptions->button(QDialogButtonBox::Reset))
    {
        ui->cbSavePasswordBook->setChecked(FileOptions.isSavePasswordBook());
        ui->cbLastOpened->setChecked(FileOptions.isRememberLastOpened());
        on_cbLastOpened_clicked();
        ui->lePathLibrary->setText(FileOptions.pathLastLibrary());
        ui->cbPathOptionsLocally->setChecked(FileOptions.isPathOptionsLocally());
        on_cbPathOptionsLocally_clicked();
        ui->lePathOptions->setText(FileOptions.pathOptions());
        ui->lePathStatistics->setText(FileOptions.pathStatistics());
        ui->cbStandardToolbar->setChecked(FileOptions.isStateStandardToolbar());
        ui->cbEditToolbar->setChecked(FileOptions.isStateEditToolbar());
        ui->cbFormatToolbar->setChecked(FileOptions.isStateFormatToolbar());
        ui->cbFindToolbar->setChecked(FileOptions.isStateFindToolbar());
        ui->lePathShelkTest->setText(FileOptions.pathProgramShelkTest());

        if (FileOptions.isLanguageProgram() == TFileOptions::LANGUAGE_RUSSIAN)
            ui->cbLanguage->setCurrentIndex(0);
        else
            ui->cbLanguage->setCurrentIndex(1);
    }
}

void TFormOptions::on_pbPathLibrary_clicked()
{
    QString str = QFileDialog::getOpenFileName(0, tr("Select Library"), ui->lePathLibrary->text(),
                                               "XWLB - " + tr("Library of books, software of Klen-library") + " (*.xwlb)");

    if(str != "") ui->lePathLibrary->setText(str);
}

void TFormOptions::on_pbPathStatistics_clicked()
{
    QString sStatistics = QFileDialog::getSaveFileName(0, tr("Select file of statistics."), ui->lePathStatistics->text(),
                                               tr("Document") + " HTML (*.html);;");

    if (sStatistics != "")
    {
        if (sStatistics.indexOf(".html") == -1) sStatistics.append(".html");

        ui->lePathStatistics->setText(sStatistics);
    }
}

void TFormOptions::on_pbPathOptions_clicked()
{
    QString str = QFileDialog::getExistingDirectory(0, tr("Selection directory path of options"), ui->lePathOptions->text());

    if(str != "") ui->lePathOptions->setText(str);
}

void TFormOptions::on_pbPathShelkTest_clicked()
{
    QString str = QFileDialog::getOpenFileName(0, tr("Select program of Shelk-test"), ui->lePathShelkTest->text());

    if(str != "") ui->lePathShelkTest->setText(str);
}
