/* ======================================================================
**  main.cpp - Исходный файл главного модуля Клён-библиотеки.
**  Является частью программы Клён-библиотека, распространяемой под
**  Стандартной общественной лицензией, версии 2 (GNU GPL v2).
** ======================================================================
**  Copyleft 2009 - 2013 by Lab Free Technologies <admin@LabFreeTech.org>
** ==============================================================================
**  Copyleft 2009 - 2013 Лаборатория свободных технологий <admin@LabFreeTech.org>
** ==============================================================================
*/

#include <QtGui/QApplication>
#include <QtGui/QIcon>
#include <QtCore/QTranslator>
#include <QtCore/QDataStream>
#include <QtCore/QTextStream>
#include <QtCore/QDir>
#include <QtCore/QStringList>
#include <QtCore/QString>
#include <QtCore/QFile>
#include <QtCore/QSettings>
#include <QtCore/QDebug>
#include <QtCore/QCryptographicHash>
#include "TKlenLibrary/TKlenLibrary.h"
#include "tfileoptions/tfileoptions.h"
#include "const.h"
#include "version.h"

//Проверка аргументов
unsigned char checkArguments(int argc, char *argv[])
{
    if (argc > 1)
    {
        if (argv[1] == QString("-H") || argv[1] == QString("--hardware"))
        {
            qDebug() << "  Hardware of developer:\n\n"
                        "  info.product = 'Intel(R) Pentium(R) 4 CPU 3.20GHz'  (string)\n"
                        "  info.product = 'G84 [GeForce 8600 GT]'  (string)\n"
                        "  input.product = 'Macintosh mouse button emulation'  (string)\n"
                        "  info.product = 'ST3250410AS'  (string)\n"
                        "  info.product = 'PIXART USB OPTICAL MOUSE'  (string)\n";;
            return 0;
        }

        if (argv[1] == QString("-v") || argv[1] == QString("--version"))
        {
            qDebug() << APPLICATION_VERSION;
            return 0;
        }

        if (argv[1] == QString("-i") || argv[1] == QString("--information"))
        {
            qDebug() << APPLICATION_NAME + " v" + APPLICATION_VERSION + ". \n"
                        "Copyleft The Laboratory of Freeware Technologies.\n"
                        "by Flegontov Denis Mikhaylovich";
            return 0;
        }

        if (argv[1] == QString("-l") || argv[1] == QString("--license"))
        {
            QFile ifile(QApplication::applicationDirPath() +  "/../share/klen-library-" +
                        APPLICATION_VERSION + "/LICENSE.txt");
            ifile.open(QIODevice::ReadOnly);
            QString sLicense;
            sLicense = ifile.readAll();
            qDebug() << sLicense;
            ifile.close();
            return 0;
        }

        if (argv[1] == QString("-S") || argv[1] == QString("--software"))
        {
            qDebug() << "  Software of developer:\n\n"
                        "  system.kernel.name = 'FreeBSD'  (string)\n"
                        "  system.kernel.version = '9.0-RELEASE'  (string)\n"
                        "  system.kernel.machine = 'amd64'  (string)\n"
                        "  info.category = 'oss'  (string)\n"
                        "  QEMU PC emulator version 0.11.1\n"
                        "  VMware Player. 4.0.3 build-703057.\n"
                        "  VirtualBox 4.1.22_OSE r80657.\n"
                        "  Qt Creator 2.6.0. Based on Qt 4.8.2 (64 bit)\n"
                        "  GIMP 2.6.12\n";
            return 0;
        }

        if (argv[1] == QString("-h") || argv[1] == QString("--help"))
        {
            qDebug() << "The \"" + APPLICATION_NAME + "\" a program for creating electronic books.\n"
                        "designed for professional creating and reading electronic books "
                        " and electronic manuals.\n"
                        "\n"
                        "The parameters:\n"
                        "\n"
                        "  -v, --version       Displays the program version.\n"
                        "  -i, --information   Displays information about the program.\n"
                        "  -l, --license       Displays the license program.\n"
                        "  -h, --help          Displays this information and exit program.\n";
            return 0;
        }
    }

    return 0;
}

//Загрузка языка установщика
unsigned char loadLanguage()
{
    QString sLanguage;
    QFile frLnguage(QDir::tempPath() + "/lang.txt");
    QTextStream tsLnguage(&frLnguage);
    tsLnguage.setCodec("UTF-8");

    if (frLnguage.open(QIODevice::ReadOnly) == true)
    {
        sLanguage = tsLnguage.readLine();
        frLnguage.close();
        QDir dFile;
        dFile.remove(frLnguage.fileName());

        if (sLanguage.indexOf("ru") != -1) return 1;
        else
        if (sLanguage.indexOf("en") != -1) return 2;
        else
            return 2;
    }

    return 0;
}

int main(int argc, char *argv[])
{
    QApplication app(argc, argv);
    checkArguments(argc, argv);//Проверка аргументов
    app.setWindowIcon(QIcon(QString::fromUtf8(":/toolbar/toolbar/klen-library.png")));

    QCoreApplication::setOrganizationName("Lab Free Technologies");
    QCoreApplication::setOrganizationDomain("http://www.en.labfreetech.org");
    QCoreApplication::setApplicationName(APPLICATION_NAME);
    QCoreApplication::setApplicationVersion(APPLICATION_VERSION);

    bool bState = false;

    TFileOptions *FileOptions = new TFileOptions;
    bState = FileOptions->load();

    QTranslator tr1, tr2;

//Загрузка языка установщика
    switch (loadLanguage())
    {
    case 1:
        FileOptions->setLanguageProgram(TFileOptions::LANGUAGE_RUSSIAN);
        break;
    case 2:
        FileOptions->setLanguageProgram(TFileOptions::LANGUAGE_ENGLISH);
        break;
    }

//Перевод программы на русский язык
    if (FileOptions->isLanguageProgram() == TFileOptions::LANGUAGE_RUSSIAN)
    {
        tr1.load(PATH_SHARE_APP + "/languages/russian/klen_ru.qm", ".");
        tr2.load(PATH_SHARE_APP + "/languages/russian/qt_ru.qm", ".");
    }

    app.installTranslator(&tr1);
    app.installTranslator(&tr2);

    TKlenLibrary FormKlenLibrary;
    FormKlenLibrary.FileOptions = FileOptions;

//Если нет файла свойств программы(первый запуск), то показывается окно приветствия
    if (!bState)
    {
        FormKlenLibrary.showWelcome();
        FileOptions->write();
    }

    FormKlenLibrary.setGeometry(FileOptions->sizeMainWindow());
    //Восстановление положений панелей инструментов
    FormKlenLibrary.restoreState(FileOptions->stateMainWindow());
    FormKlenLibrary.getUserName();
    FormKlenLibrary.show();
    FormKlenLibrary.setVisibilityOfToolbar();

//Применение режима работы программы
    switch(FileOptions->isModeWork())
    {
    case TFileOptions::MODE_READING:
        FormKlenLibrary.ui->actionEditing->setChecked(false);
        FormKlenLibrary.ui->actionReading->setChecked(true);
        FormKlenLibrary.slotModeViewing();
        break;

    case TFileOptions::MODE_EDITING:
        FormKlenLibrary.ui->actionEditing->setChecked(true);
        FormKlenLibrary.ui->actionReading->setChecked(false);
        FormKlenLibrary.slotModeEditing();
        break;
    }

    if (FileOptions->isRememberLastOpened() == true) FormKlenLibrary.openLibrary(true);
    if (FileOptions->pathProgramShelkTest().isEmpty()) FormKlenLibrary.ui->actionTest->setEnabled(false);

    return app.exec();
}
