/* ======================================================================
**  tfileoptions.cpp - Исходный файл модуля свойств программы.
**  Является частью программы Шёлковый тест, распространяемой под
**  Стандартной общественной лицензией, версии 2 (GNU GPL v2).
** ======================================================================
**  Copyleft 2009 - 2013 by Lab Free Technologies <admin@LabFreeTech.org>
** ==============================================================================
**  Copyleft 2009 - 2013 Лаборатория свободных технологий <admin@LabFreeTech.org>
** ==============================================================================
*/

#include "tfileoptions.h"

TFileOptions::TFileOptions() : b_ModeWork(MODE_READING),
    b_LanguageProgram(LANGUAGE_RUSSIAN),
    b_StateStandardToolbar(TOOLBAR_SHOW),
    b_StateEditToolbar(TOOLBAR_HIDE),
    b_StateFormatToolbar(TOOLBAR_HIDE),
    b_StateFindToolbar(TOOLBAR_HIDE),
    b_PathOptionsLocally(true),
    b_RememberLastOpened(true),
    b_SavePasswordBook(true),
    ba_ModeEditingPassword(""),
    s_PathStatistics(QDir::homePath()),
    s_PathOptions(""),
    s_PathLastLibrary(""),
    s_PathProgramShelkTest(""),
    r_SizeMainWindow(QRect(260, 300, 700, 420))
{

}

TFileOptions::~TFileOptions()
{
    clear();
}

bool TFileOptions::isModeWork() const
{
    return b_ModeWork;
}

bool TFileOptions::isLanguageProgram() const
{
    return b_LanguageProgram;
}

bool TFileOptions::isStateStandardToolbar() const
{
    return b_StateStandardToolbar;
}

bool TFileOptions::isStateEditToolbar() const
{
    return b_StateEditToolbar;
}

bool TFileOptions::isStateFormatToolbar() const
{
    return b_StateFormatToolbar;
}

bool TFileOptions::isStateFindToolbar() const
{
    return b_StateFindToolbar;
}

bool TFileOptions::isPathOptionsLocally() const
{
    return b_PathOptionsLocally;
}

bool TFileOptions::isRememberLastOpened() const
{
    return b_RememberLastOpened;
}

bool TFileOptions::isSavePasswordBook() const
{
    return b_SavePasswordBook;
}

QByteArray TFileOptions::modeEditingPassword() const
{
    return ba_ModeEditingPassword;
}

QString TFileOptions::pathStatistics() const
{
    return s_PathStatistics;
}

QString TFileOptions::pathOptions() const
{
    return s_PathOptions;
}

QString TFileOptions::pathLastLibrary() const
{
    return s_PathLastLibrary;
}

QString TFileOptions::pathProgramShelkTest() const
{
    return s_PathProgramShelkTest;
}

QRect TFileOptions::sizeMainWindow() const
{
    return r_SizeMainWindow;
}

QByteArray TFileOptions::stateMainWindow() const
{
    return ba_StateMainWindow;
}

void TFileOptions::setModeWork(bool bModeWork)
{
    b_ModeWork = bModeWork;
}

void TFileOptions::setLanguageProgram(bool bLanguageProgram)
{
    b_LanguageProgram = bLanguageProgram;
}

void TFileOptions::setStateStandardToolbar(bool bStateStandardToolbar)
{
    b_StateStandardToolbar = bStateStandardToolbar;
}

void TFileOptions::setStateEditToolbar(bool bStateEditToolbar)
{
    b_StateEditToolbar = bStateEditToolbar;
}

void TFileOptions::setStateFormatToolbar(bool bStateFormatToolbar)
{
    b_StateFormatToolbar = bStateFormatToolbar;
}

void TFileOptions::setStateFindToolbar(bool bStateFindToolbar)
{
    b_StateFindToolbar = bStateFindToolbar;
}

void TFileOptions::setPathOptionsLocally(bool bPathOptionsLocally)
{
    b_PathOptionsLocally = bPathOptionsLocally;
}

void TFileOptions::setRememberLastOpened(bool bRememberLastOpened)
{
    b_RememberLastOpened = bRememberLastOpened;
}

void TFileOptions::setSavePasswordBook(bool bSavePasswordBook)
{
    b_SavePasswordBook = bSavePasswordBook;
}

void TFileOptions::setModeEditingPassword(QByteArray baModeEditingPassword)
{
    ba_ModeEditingPassword = baModeEditingPassword;
}

void TFileOptions::setPathStatistics(QString sPathStatistics)
{
    s_PathStatistics = sPathStatistics;
}

void TFileOptions::setPathOptions(QString sPathOptions)
{
    s_PathOptions = sPathOptions;
}

void TFileOptions::setPathLastLibrary(QString sPathLastLibrary)
{
    s_PathLastLibrary = sPathLastLibrary;
}

void TFileOptions::setPathProgramShelkTest(QString sPathProgramShelkTest)
{
    s_PathProgramShelkTest = sPathProgramShelkTest;
}

void TFileOptions::setSizeMainWindow(QRect rSizeMainWindow)
{
    r_SizeMainWindow = rSizeMainWindow;
}

void TFileOptions::setStateMainWindow(QByteArray baStateMainWindow)
{
    ba_StateMainWindow = baStateMainWindow;
}

bool TFileOptions::load()
{
    bool bState = false;
    QByteArray baPassword;

    QSettings *setFileOptions = new QSettings(QSettings::IniFormat, QSettings::UserScope, QCoreApplication::organizationName(), QCoreApplication::applicationName());
    setFileOptions->beginGroup("/ProgramData");
    setPathOptionsLocally(setFileOptions->value("/bPathOptionsLocally", isPathOptionsLocally()).toBool());
    setPathOptions(setFileOptions->value("/sPathOptions", pathOptions()).toString());

    if (!isPathOptionsLocally() && !pathOptions().isEmpty())
    {
        delete setFileOptions;
        setFileOptions = new QSettings(pathOptions() + "/Klen-library.ini", QSettings::IniFormat);
        setFileOptions->beginGroup("/ProgramData");
    }

    setModeWork(setFileOptions->value("/bModeWork", isModeWork()).toBool());
    setLanguageProgram(setFileOptions->value("/bLanguageProgram", isLanguageProgram()).toBool());
    setStateStandardToolbar(setFileOptions->value("/bStateStandardToolbar", isStateStandardToolbar()).toBool());
    setStateEditToolbar(setFileOptions->value("/bStateEditToolbar", isStateEditToolbar()).toBool());
    setStateFormatToolbar(setFileOptions->value("/bStateFormatToolbar", isStateFormatToolbar()).toBool());
    setStateFindToolbar(setFileOptions->value("/bStateFindToolbar", isStateFindToolbar()).toBool());
    setModeEditingPassword(setFileOptions->value("/baModeEditingPassword", baPassword).toByteArray());
    setPathStatistics(setFileOptions->value("/sPathStatistics", pathStatistics()).toString());
    setPathProgramShelkTest(setFileOptions->value("/sPathProgramShelkTest", pathProgramShelkTest()).toString());
    setFileOptions->endGroup();
    setFileOptions->beginGroup("/ProgramView");
    setSizeMainWindow(setFileOptions->value("/rSizeMainWindow", sizeMainWindow()).toRect());
    setStateMainWindow(setFileOptions->value("/baStateMainWindow", stateMainWindow()).toByteArray());
    setFileOptions->endGroup();
    setFileOptions->beginGroup("/Library");
    setRememberLastOpened(setFileOptions->value("/bRememberLastOpened", isRememberLastOpened()).toBool());
    setPathLastLibrary(setFileOptions->value("/sPathLastLibrary", pathLastLibrary()).toString());
    setSavePasswordBook(setFileOptions->value("/bSavePasswordBook", isSavePasswordBook()).toBool());
    setFileOptions->endGroup();

    if (QFile::exists(setFileOptions->fileName()))
        bState = true;
    else
        bState = false;

    delete setFileOptions;

    return bState;
}

bool TFileOptions::write()
{
    QSettings setFileOptions(QSettings::IniFormat, QSettings::UserScope, QCoreApplication::organizationName(), QCoreApplication::applicationName());

    setFileOptions.beginGroup("/ProgramData");
    setFileOptions.setValue("/bPathOptionsLocally", isPathOptionsLocally());
    setFileOptions.setValue("/sPathOptions", pathOptions());
    setFileOptions.setValue("/bModeWork", isModeWork());
    setFileOptions.setValue("/bLanguageProgram", isLanguageProgram());
    setFileOptions.setValue("/bStateStandardToolbar", isStateStandardToolbar());
    setFileOptions.setValue("/bStateEditToolbar", isStateEditToolbar());
    setFileOptions.setValue("/bStateFormatToolbar", isStateFormatToolbar());
    setFileOptions.setValue("/bStateFindToolbar", isStateFindToolbar());
    setFileOptions.setValue("/baModeEditingPassword", modeEditingPassword());
    setFileOptions.setValue("/sPathStatistics", pathStatistics());
    setFileOptions.setValue("/sPathProgramShelkTest", pathProgramShelkTest());
    setFileOptions.endGroup();
    setFileOptions.beginGroup("/ProgramView");
    setFileOptions.setValue("/rSizeMainWindow", sizeMainWindow());
    setFileOptions.setValue("/baStateMainWindow", stateMainWindow());
    setFileOptions.endGroup();
    setFileOptions.beginGroup("/Library");
    setFileOptions.setValue("/bRememberLastOpened", isRememberLastOpened());
    setFileOptions.setValue("/sPathLastLibrary", pathLastLibrary());
    setFileOptions.setValue("/bSavePasswordBook", isSavePasswordBook());
    setFileOptions.endGroup();
    setFileOptions.sync();

    return true;
}

void TFileOptions::clear()
{
    setModeWork(MODE_READING);
    setLanguageProgram(LANGUAGE_RUSSIAN);
    setStateStandardToolbar(TOOLBAR_SHOW);
    setStateEditToolbar(TOOLBAR_HIDE);
    setStateFormatToolbar(TOOLBAR_HIDE);
    setStateFindToolbar(TOOLBAR_HIDE);
    setSavePasswordBook(true);
    setPathOptionsLocally(true);
    setRememberLastOpened(true);
    ba_ModeEditingPassword.clear();
    s_PathStatistics.clear();
    s_PathOptions.clear();
    s_PathLastLibrary.clear();
    s_PathProgramShelkTest.clear();
    r_SizeMainWindow = QRect(260, 300, 700, 420);
}
